/*

  Modstands Måler med LCD Display 1602 I2C Interface

  Dele krævet:
  - Arduino Board
  - 1602 LCD Display I2C Interface
  - 10K Modstand
  - Breadboard
  - Kabler

  Biblioteker:
  - LiquidCrystal_I2C

  Programmering af : René Rasmussen
  Webside : www.it-teknolog.dk
  kontakt : renerasmussen@it-teknolog.dk
  Dato : 19 Aug 2019
  Version : v1.01

  Information:
  Programmet måler test modstanden med hjælp fra en spændingsdeler, og udregner værdien med
  ohms lov, hvis der ikke er isat en modstand, viser skærmen INGEN MODSTAND, programmet køre
  i en loop, så man kan skifte modstand mens programmet kører.

  Når du kører projektet, kan det være at det ikke helt virker som det skal at den siger 
  INGEN MODSTAND selvom der er monteret en test modtsnad, grunden til dette kan være
  at modstanden i dit kredsløb er højre eller lavere, end det som projektet er lavet efter,
  derfor kan det være nødvendig at fin tune lidt på de forskellige værdier for hver modstand.

  Hvis du har forbedringer eller finder fejl eller mangler er du velkommen til at skrive 
  på ovenstående email. 
  
  Og selvfølgelig gerne ris og ros.

*/

#include <LiquidCrystal_I2C.h>        // inkludere bibliotek LiquidCrystal_I2C.h

LiquidCrystal_I2C lcd(0x27, 16, 2);   // opret objekt fra LiquidCrystal_I2C til lcd

float Uin = 5;                        // angiver Uin = 5v fra Arduino
float Uout = 0;                       // Uout er spændingen over R1
float Uread = 0;                      // angiver Uread
float R1 = 10000;                     // modstand R1
long R2 = 0;                          // modstand R2
String result = "";                   // resultat som skrives til LCD display

void setup() {
  lcd.init();                         // start LCD Display
  lcd.backlight();                    // tænd baggrundslys
}

void loop() {
  Serial.begin(9600);                 // start serial kommunikation

  Uread = analogRead(A0);             // læs A0 (0-1023) og gem i variabel Uread
  Uout = Uread * 0.004885;            // beregn Uout 
  R2 = (Uin / Uout) * R1 - R1;        // spændingsdeler udregning og resultat R2

  if (R2 <= -1) {                     // hvis modstanden er under 0
    result = " INGEN MODSTAND";       // skriv til LCD display 
    Serial.println("INGEN MODSTAND"); // skriv INGEN MODSTAND til serial monitoren
  }
  else{
    Serial.println(R2);               // skriv R2 til serial monitoren for nøjagtig resultat
  }

  // --- resultater for modstande ---
  if (R2 >= 5 && R2 <= 20) { result = "10 Ohm"; } 
  if (R2 >= 90 && R2 <= 120) { result = "100 Ohm"; }
  if (R2 >= 130 && R2 <= 180) { result = "150 Ohm"; }  
  if (R2 >= 970 && R2 <= 1030) { result = "1K Ohm"; }
  if (R2 >= 9900 && R2 <= 10100) { result = "10K Ohm"; }
  if (R2 >= 100000 && R2 <= 105000) { result = "100K Ohm"; }
  if (R2 >= 990000 && R2 <= 1500000) { result = "1M Ohm"; }

  if (R2 >= 21 && R2 <= 45) { result = "22 Ohm"; } 
  if (R2 >= 200 && R2 <= 210) { result = "200 Ohm"; }
  if (R2 >= 215 && R2 <= 250) { result = "220 Ohm"; }
  if (R2 >= 255 && R2 <= 290) { result = "270 Ohm"; }  
  if (R2 >= 1900 && R2 <= 2090) { result = "2K Ohm"; }
  if (R2 >= 2100 && R2 <= 2300) { result = "2K2 Ohm"; }
  if (R2 >= 20300 && R2 <= 20500) { result = "20K Ohm"; }
  if (R2 >= 232000 && R2 <= 235000) { result = "200K Ohm"; }

  if (R2 >= 320 && R2 <= 370) { result = "330 Ohm"; }
  if (R2 >= 3280 && R2 <= 3350) { result = "3K3 Ohm"; }
  if (R2 >= 299000 && R2 <= 335000) { result = "300K Ohm"; }

  if (R2 >= 40 && R2 <= 60) { result = "47 Ohm"; } 
  if (R2 >= 460 && R2 <= 500) { result = "470 Ohm"; }
  if (R2 >= 4500 && R2 <= 4800) { result = "4K7 Ohm"; }
  if (R2 >= 46000 && R2 <= 48000) { result = "47K Ohm"; }
  if (R2 >= 460000 && R2 <= 570000) { result = "470K Ohm"; }
      
  if (R2 >= 500 && R2 <= 530) { result = "510 Ohm"; }
  if (R2 >= 5050 && R2 <= 5200) { result = "5K1 Ohm"; }
  if (R2 >= 50500 && R2 <= 52000) { result = "51K Ohm"; } 

  if (R2 >= 600 && R2 <= 750) { result = "680 Ohm"; }
  if (R2 >= 6600 && R2 <= 6900) { result = "6K8 Ohm"; }
  if (R2 >= 67000 && R2 <= 69000) { result = "68K Ohm"; }
  if (R2 >= 600000 && R2 <= 900000) { result = "680K Ohm"; }
  
  lcd.setCursor(0, 0);                // sæt cursor til første position og første linie
  lcd.print(result);                  // skriv resultater på LCD display
  lcd.print("           ");           // slet yderligere positioner
  delay(100);                         // vent 100ms
}
